<?php

namespace SuperRhino\Utils;

use Psr\Container\ContainerInterface;
use SuperRhino\Utils\Container\Container;

abstract class Facade {

    /**
     * @var ContainerInterface
     */
    protected static $container;

    /**
     * The resolved object instances.
     *
     * @var array
     */
    protected static $resolvedInstance;

    static function register(ContainerInterface $container) {
        self::$container = new Container($container);
    }

    /**
     * @return ContainerInterface
     */
    static function getContainer() {
        return self::$container;
    }

    /**
     * Get the registered name of the component.
     *
     * @return string|object
     *
     * @throws \RuntimeException
     */
    protected static function getFacadeAccessor() {
        throw new \RuntimeException(sprintf('Facade "%s" does not implement getFacadeAccessor method.', static::class));
    }

    /**
     * Resolve the facade root instance from the container.
     *
     * @param object|string $name
     * @return mixed
     */
    protected static function resolveFacadeInstance($name) {
        if (is_object($name)) {
            return $name;
        }

        if (isset(static::$resolvedInstance[$name])) {
            return static::$resolvedInstance[$name];
        }

        if (static::$container) {
            return static::$resolvedInstance[$name] = static::$container->get($name);
        }
    }

    /**
     * Get the root object behind the facade.
     *
     * @return mixed
     */
    public static function getFacadeRoot() {
        return static::resolveFacadeInstance(static::getFacadeAccessor());
    }

    /**
     * Handle dynamic, static calls to the object.
     *
     * @param string $method
     * @param array $args
     * @return mixed
     *
     * @throws \RuntimeException
     */
    public static function __callStatic($method, $args) {
        $instance = static::getFacadeRoot();

        if (!$instance) {
            throw new \RuntimeException(sprintf('A facade "%s" root has not been set.', static::class));
        }

        //return call_user_func_array([$instance, $method], $args);
        return $instance->$method(...$args);
    }
}